/* Distributed Checksum Clearinghouse
 *
 * test dccifd via dccif()
 *
 * Copyright (c) 2004 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.36-1.5 $Revision$
 */

#include "dccif.h"
#include "dcc_clnt.h"
#include <arpa/inet.h>

static void
usage(void)
{
	fputs("usage: [-h homedir | hostname,port] [-o opts]\n"
	      "    [-c clnt-name] [-l heLo]"
	      " [-f env_from] [-I ifile] [-O ofile]\n"
	      "    [-r rcpt1[,user1]] [-r rcpt2[,user2] [...]\n",
	      stderr);
	exit(1);
}



int NRATTRIB
main(int argc, char **argv)
{
	DCC_EMSG emsg;
	const char *homedir;
	const char *opts;
	const struct hostent *hp;
	DCC_SOCKU su, *sup;
	const char *clnt_nm;
	const char *helo;
	const char *env_from;
	DCCIF_RCPT *rcpts, *rcpt;
	int in_body_fd, out_body_fd;
	u_char result;
	char *p;
	int i, error;

	homedir = 0;
	opts = 0;
	sup = 0;
	clnt_nm = 0;
	helo = 0;
	env_from = 0;
	in_body_fd = 0;
	out_body_fd = dup(1);
	rcpts = 0;

	while (EOF != (i = getopt(argc, argv, "h:o:c:l:f:I:O:r:"))) {
		switch (i) {
		case 'h':
			homedir = optarg;
			break;

		case 'o':
			/* convert commas that users like to type to blanks */
			while ((p = strchr(optarg, ',')) != 0)
				*p = ' ';
			opts = optarg;
			break;

		case 'c':
			clnt_nm = optarg;
			dcc_host_lock();
			hp = dcc_get_host(clnt_nm, 2, &error);
			if (!hp) {
				fprintf(stderr,
					"dcc_get_host(%s): %s\n", clnt_nm,
					hstrerror(error));
				exit(1);
			}
			dcc_mk_su(&su, hp->h_addrtype, hp->h_addr, 0);
			sup = &su;
			break;

		case 'l':
			helo = optarg;
			break;

		case 'f':
			env_from = optarg;
			break;

		case 'I':
			close(in_body_fd);
			in_body_fd = open(optarg, O_RDONLY, 0);
			if (in_body_fd < 0) {
				fprintf(stderr, "open(%s): %s\n",
					optarg, strerror(errno));
				exit(1);
			}
			break;

		case 'O':
			close(out_body_fd);
			out_body_fd = open(optarg, O_WRONLY, 0666);
			if (out_body_fd < 0) {
				fprintf(stderr, "open(%s): %s\n",
					optarg, strerror(errno));
				exit(1);
			}
			break;

		case 'r':
			rcpt = malloc(sizeof(*rcpt));
			memset(rcpt, 0, sizeof(*rcpt));
			rcpt->next = rcpts;
			rcpt->addr = optarg;
			p = strchr(optarg, ',');
			if (!p) {
				rcpt->user = "";
			} else {
				*p++ = '\0';
				rcpt->user = p;
			}
			rcpt->ok = '?';
			rcpts = rcpt;
			break;

		default:
			usage();
			break;
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 0)
		usage();

	result = dccif(emsg,
		       out_body_fd, 0,
		       opts, sup, clnt_nm, helo, env_from,
		       rcpts,
		       in_body_fd, 0,
		       homedir);
	if (!result) {
		printf("result 0: %s\n", emsg);
		exit(0);
	}
	printf("overall result = %c\n", result);
	for (rcpt = rcpts; rcpt; rcpt = rcpt->next) {
		printf("    %s %s: %c\n", rcpt->addr, rcpt->user, rcpt->ok);
	}
	putchar('\n');
	exit(0);
}
