/* Distributed Checksum Clearinghouse
 *
 * Copyright (c) 2004 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.36-1.8 $Revision$
 */

#include "dcc_defs.h"


/* because tv.tv_sec is not a time_t on all systems
 * and to be thread safe on WIN32 */
const struct tm *
dcc_localtime(time_t secs, struct tm *result)
{
#ifdef HAVE_LOCALTIME_R
	return localtime_r(&secs, result);
#else
	dcc_localtime_lock();
	result = localtime(&secs);
	dcc_localtime_unlock();
	return result;
#endif
}



int
dcc_get_secs(char *s, char **end, int min, int max, int def)
{
	static time_t epoch = 0;
	char *p;
	int secs;

	if (*s == '\0' || *s == ',') {
		secs = def;
		p = s;
	} else if (min > 0
		   && !strncasecmp(s, "never", 5)) {
		if (!epoch)
			epoch = time(0);
		secs = epoch;
		p = s+5;
	} else if ((secs = strtoul(s, &p, 0)) >= DCC_MAX_SECS/60) {
		return -1;
	} else if (*p == '\0' || *p == ',') {
		;
	} else if (!strncasecmp(p, "seconds", 7)) {
		p += 7;
	} else if (!strncasecmp(p, "s", 1)) {
		++p;

	} else if (!strncasecmp(p, "minutes", 7)) {
		secs *= 60;
		p += 7;
	} else if (!strncasecmp(p, "minute", 6)) {
		secs *= 60;
		p += 6;
	} else if (!strncasecmp(p, "m", 1)) {
		secs *= 60;
		++p;

	} else if (secs >= DCC_MAX_SECS/(60*60)) {
		return -1;
	} else if (!strncasecmp(p, "hours", 5)) {
		secs *= 60*60;
		p += 5;
	} else if (!strncasecmp(p, "hour", 4)) {
		secs *= 60*60;
		p += 4;
	} else if (!strncasecmp(p, "h", 1)) {
		secs *= 60*60;
		++p;

	} else if (secs >= DCC_MAX_SECS/(24*60*60)) {
		return -1;
	} else if (!strncasecmp(p, "days", 4)) {
		secs *= 24*60*60;
		p += 4;
	} else if (!strncasecmp(p, "day", 3)) {
		secs *= 24*60*60;
		p += 3;
	} else if (!strncasecmp(p, "d", 1)) {
		secs *= 24*60*60;
		++p;

	} else if (secs >= DCC_MAX_SECS/(7*24*60*60)) {
		return -1;
	} else if (!strncasecmp(p, "weeks", 5)) {
		   secs *= 7*24*60*60;
		   p += 5;
	} else if (!strncasecmp(p, "week", 4)) {
		secs *= 7*24*60*60;
		p += 4;
	} else if (!strncasecmp(p, "w", 1)) {
		   secs *= 7*24*60*60;
		   ++p;

	} else {
		return -1;
	}

	if (secs > max)
		return -1;
	if (secs < min && secs != 0)
		return -1;

	if (*p != '\0') {
		if (*p != ',' || !end)
			return -1;
		++p;
	}
	if (end)
		*end = p;
	return secs;
}
