/* Distributed Checksum Clearinghouse
 *
 * white-list lister
 *
 * Copyright (c) 2004 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.36-1.20 $Revision$
 */

#include "dcc_defs.h"
#include "dcc_ck.h"

static DCC_EMSG dcc_emsg;
static DCC_WF cmn_wf;
static const char *homedir;

static void NRATTRIB
usage(void)
{
	dcc_logbad(EX_USAGE, "usage: [-V] [-h homedir] [-t env_to] file");
}


int
main(int argc, char **argv)
{
	const char *to = 0;
	u_char print_version = 0;
	DCC_WHITE_INX inx;
	DCC_WHITE_ENTRY *e;
	const char *nm;
	DCC_SUM sum;
	char tgts_buf[20], time_buf[26];
	struct tm tm;
	int i;

	dcc_syslog_init(0, 0);
	dcc_clnt_debug = 99;

	dup2(1, 2);			/* put error messages in context */

	while ((i = getopt(argc, argv, "Vh:t:")) != EOF) {
		switch (i) {
		case 'V':
			fprintf(stderr, DCC_VERSION"\n");
			print_version = 1;
			break;

		case 'h':
			homedir = optarg;
			break;

		case 't':
			to = optarg;
			break;

		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;
	if (argc == 0) {
		if (print_version)
			exit(EX_OK);
		usage();
	}

	if (!dcc_cdhome(dcc_emsg, homedir))
		dcc_error_msg("%s", dcc_emsg);

	dcc_wf_init(&cmn_wf, 0, 0);

	while ((nm = *argv++) != 0) {
		printf("\n%s\n", nm);
		if (!dcc_white_rdy(dcc_emsg, &cmn_wf, nm, 1, 0)) {
			dcc_wf_unlock(&cmn_wf);
			dcc_error_msg("%s", dcc_emsg);
			exit(EX_DATAERR);
		}

		if (to) {
			dcc_str2ck(sum, 0, 0, to);
			printf("%s\n%8s %s\n    ",
			       to,
			       dcc_type2str_err(DCC_CK_ENV_TO, 0, 0),
			       dcc_ck2str_err(DCC_CK_ENV_TO, sum));
			switch (dcc_white_sum(dcc_emsg, &cmn_wf,
					      DCC_CK_ENV_TO, sum, 1)) {
			case DCC_WHITE_ERROR:
				dcc_error_msg("%s", dcc_emsg);
				break;
			case DCC_WHITE_HALF_LISTED:
				printf("DCC_WHITE_HALF_LISTED\n");
				break;
			case DCC_WHITE_LISTED:
				printf("DCC_WHITE_LISTED\n");
				break;
			case DCC_WHITE_UNLISTED:
				printf("DCC_WHITE_UNLISTED\n");
				break;
			case DCC_WHITE_BLACK:
				printf("DCC_WHITE_BLACK\n");
				break;
			}
			continue;
		}

		printf("    %d entries  %d gen",
		       cmn_wf.info->hdr.entries, cmn_wf.info->hdr.gen);
		switch (cmn_wf.info->hdr.resolve) {
		case DCC_WHITE_RESOLVE_BAD:
			printf("  hash table broken\n");
			break;
		case DCC_WHITE_RESOLVE_NO_IP:
			printf("  no IP addresses\n");
			break;
		default:
			strftime(time_buf, sizeof(time_buf), "%b %d %X %Z",
				 dcc_localtime(cmn_wf.info->hdr.resolve, &tm));
			printf("  resolve hostnames after %s\n", time_buf);
			break;
		}
		for (i = 0; i < DIM(cmn_wf.info->hdr.white_incs); ++i) {
			if (cmn_wf.info->hdr.white_incs[i][0] == '\0')
				break;
			if (!i)
				printf("    includes\n");
			printf("      %s\n",
			       DCC_NM2PATH(cmn_wf.info->hdr.white_incs[i]));
		}
		putchar('\n');

		/* first the hash table */
		for (inx = 0; inx < DIM(cmn_wf.info->bins); ++inx) {
			printf("%4d:", inx);
			if (cmn_wf.info->bins[inx])
				printf("%-4d", cmn_wf.info->bins[inx]);
			else
				printf("    ");
			putchar((inx % 4) == 3 ? '\n' : '\t');
		}
		fputs("\n\n", stdout);

		/* then the entries */
		for (inx = 0; inx < cmn_wf.info_entries; ++inx) {
			e = &cmn_wf.info->tbl[inx];
			if (e->type == DCC_CK_INVALID)
				continue;
			printf("%4d->%-4d  %12s  %6s  %s\n",
			       inx, e->fwd,
			       dcc_type2str_err(e->type, 0, 0),
			       dcc_cnt2str(tgts_buf, sizeof(tgts_buf),
					   e->tgts, 0),
			       dcc_ck2str_err(e->type, e->sum));
		}
	}
	exit(EX_OK);
}
